/*
 * Project name:
     Thumbstick click - Thumbstick Click demo
 * Copyright:
     (c) mikroElektronika, 2015.
 * Revision History:
     20140723:
       - Initial release (DR & TP);
 * Description:
     This is a simple project which demonstrates the use of Thumbstick click board.
     Input device for 360 degree directional movement. TFT shows values.
     The joystick unit receives directional input in the x and y axes by way of two potentiometers connected to a stick. 
     The mechanical pushbutton sends an interrupt signal to the microcontroller. 
     Thumbstick click communicates with the target board through mikroBUS SPI (CS, SCK, MISO and MOSI) lines. 
     The board is designed to use either a 3.3V or a 5V power supply.
 * Test configuration:
     MCU:             P32MX795F512L
                      http://ww1.microchip.com/downloads/en/DeviceDoc/61156F.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     ext. modules:    Thumbstick click
                      http://www.mikroe.com/click/thumbstick/
                      EasyTFT Board
                      http://www.mikroe.com/eng/products/view/277/various-components/
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 * NOTES:
     - Place Thumbstick Click board in the mikroBUS socket 1.
     - Place EasyTFT and turn on TFT backlight switch SW11.1.
 */

#include "resources.h"

// TFT module connections
char TFT_DataPort at LATE;
sbit TFT_RST at LATD7_bit;
sbit TFT_BLED at LATD2_bit;
sbit TFT_RS at LATD9_bit;
sbit TFT_CS at LATD10_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISD7_bit;
sbit TFT_BLED_Direction at TRISD2_bit;
sbit TFT_RS_Direction at TRISD9_bit;
sbit TFT_CS_Direction at TRISD10_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End TFT module connections

// Thumbstick click module connections
sbit Chip_Select           at RC2_bit;
sbit Test                  at RD0_bit;
sbit Button                at RE8_bit;
sbit Chip_Select_Direction at TRISC2_bit;
sbit Test_Direction        at TRISD0_bit;
sbit Button_Direction      at TRISE8_bit;

unsigned int left_right, for_rev, old_left_right, old_for_rev;
unsigned int middle_left_right, middle_for_rev;

char txt[10];

void int1_interrupt() iv IVT_EXTERNAL_1 ilevel 4 ics ICS_AUTO {
  INT1IF_bit = 0;
  Test = 1;                   // test
}

void InitDisplay(){
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_BLED = 1;
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("Thumbstick  Click", 75, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyPIC Fusion v7", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);

}

// Get ADC values
unsigned int GetADC(unsigned short channel) {    // Returns 0..4095
  unsigned int tmp;
  Chip_Select = 0;                               // Select MCP3204
  SPI3_Write(0x06);                              // SPI communication using 8-bit segments
  channel = channel << 6;                        // Bits 7 & 6 define ADC input
  tmp = SPI3_Read(channel) & 0x0F;               // Get first 8 bits of ADC value
  tmp = tmp << 8;                                // Shift ADC value by 8
  tmp = tmp | SPI3_Read(0);                      // Get remaining 4 bits of ADC value
                                                 // and form 12-bit ADC value
  Chip_Select= 1;                                // Deselect MCP3204
  return tmp;                                    // Returns 12-bit ADC value
}

void DisplayValues() {
  int tmp;
  if(old_left_right != left_right) {
    TFT_Rectangle(70,60,240,90);                 // Clear area
    tmp = middle_left_right - left_right;
    if(tmp < 0) {                                //move right
      WordToStr((unsigned int)(left_right - middle_left_right),&txt);
      TFT_Write_Text("Right:",80,65);
    }else{                                       //move left
      WordToStr((unsigned int)(middle_left_right - left_right),&txt);
      TFT_Write_Text("Left:",80,65);
    }
    TFT_Write_Text(txt,180,65);
    old_left_right = left_right;
  }
  
  if(old_for_rev != for_rev) {
    TFT_Rectangle(70,120,240,150);
    tmp = middle_for_rev - for_rev;
    if(tmp < 0) {                               //move reverse
      WordToStr((unsigned int)(for_rev - middle_for_rev),&txt);
      TFT_Write_Text("Reverse:",80,125);
    }else{                                      //move forward
      WordToStr((unsigned int)(middle_for_rev - for_rev),&txt);
      TFT_Write_Text("Forward:",80,125);
    }
    TFT_Write_Text(txt,180,125);
    old_for_rev = for_rev;
  }

}

void main() {
  AD1PCFG     = 0xFFFF;           // Turning all analog pins to digital

   // Set Chip Select pin as Output
  Chip_Select_Direction = 0;
  Test_Direction        = 0;
  Button_Direction      = 1;

  // SPI config for Thumbstick click
  SPI3_Init();
                     
  InitDisplay();

  //Set middle values
  middle_for_rev = GetADC(0);
  middle_left_right = GetADC(1);

  INT1IP0_bit = 0;                   // Set INT1 interrupt
  INT1IP1_bit = 0;                   // Set interrupt priorities
  INT1IP2_bit = 1;                   // Set inrrupt priority to 4
  INT1IE_bit  = 1;                   // Set interrupt on INT1 (RE8) to be enabled
  INT1EP_bit  = 1;                   // Interrupt on rising edge
  EnableInterrupts();                // Enable interruts as previously set

  while(1) {
     for_rev = GetADC(0);
     left_right = GetADC(1);
     DisplayValues();

     if (Button == 0) {
         TFT_Rectangle(70,170,240,200);
         TFT_Write_Text("Button   Pressed",100,177);
       }
       else {
         TFT_Rectangle(72,172,238,198);
         TFT_Write_Text("Button   Released",100,177);
       }
     if (Test && Button) {
       Test = 0;
     }
     delay_ms(10);
  }
}